@@ -20,7 +20,13 @@ GsApp *
 gs_appstream_create_app (GsPlugin *plugin, XbSilo *silo, XbNode *component, GError **error)
 {
 	GsApp *app;
-	g_autoptr(GsApp) app_new = gs_app_new (NULL);
+	g_autoptr(GsApp) app_new = NULL;
+
+	/* The 'plugin' can be NULL, when creating app for --show-metainfo */
+	g_return_val_if_fail (XB_IS_SILO (silo), NULL);
+	g_return_val_if_fail (XB_IS_NODE (component), NULL);
+
+	app_new = gs_app_new (NULL);
 
 	/* refine enough to get the unique ID */
 	if (!gs_appstream_refine_app (plugin, app_new, silo, component,
@@ -33,6 +39,9 @@ gs_appstream_create_app (GsPlugin *plugin, XbSilo *silo, XbNode *component, GErr
 	if (gs_app_has_quirk (app_new, GS_APP_QUIRK_IS_WILDCARD) || plugin == NULL)
 		return g_steal_pointer (&app_new);
 
+	if (plugin == NULL)
+		return g_steal_pointer (&app_new);
+
 	/* look for existing object */
 	app = gs_plugin_cache_lookup (plugin, gs_app_get_unique_id (app_new));
 	if (app != NULL)
@@ -350,6 +359,7 @@ gs_appstream_refine_add_addons (GsPlugin *plugin,
 	g_autofree gchar *xpath = NULL;
 	g_autoptr(GError) error_local = NULL;
 	g_autoptr(GPtrArray) addons = NULL;
+	g_autoptr(GsAppList) addons_list = NULL;
 
 	/* get all components */
 	xpath = g_strdup_printf ("components/component/extends[text()='%s']/..",
@@ -358,24 +368,34 @@ gs_appstream_refine_add_addons (GsPlugin *plugin,
 	if (addons == NULL) {
 		if (g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
 			return TRUE;
-		if (g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT))
-			return TRUE;
 		g_propagate_error (error, g_steal_pointer (&error_local));
 		return FALSE;
 	}
+
+	addons_list = gs_app_list_new ();
+
 	for (guint i = 0; i < addons->len; i++) {
 		XbNode *addon = g_ptr_array_index (addons, i);
-		g_autoptr(GsApp) app2 = NULL;
-		app2 = gs_appstream_create_app (plugin, silo, addon, error);
-		if (app2 == NULL)
+		g_autoptr(GsApp) addon_app = NULL;
+
+		addon_app = gs_appstream_create_app (plugin, silo, addon, error);
+		if (addon_app == NULL)
 			return FALSE;
-		gs_app_add_addon (app, app2);
+
+		gs_app_list_add (addons_list, addon_app);
 	}
+
+	gs_app_add_addons (app, addons_list);
+
 	return TRUE;
 }
 
 static gboolean
-gs_appstream_refine_add_images (GsApp *app, AsScreenshot *ss, XbNode *screenshot, GError **error)
+gs_appstream_refine_add_images (GsApp *app,
+				AsScreenshot *ss,
+				XbNode *screenshot,
+				gboolean *out_any_added,
+				GError **error)
 {
 	g_autoptr(GError) error_local = NULL;
 	g_autoptr(GPtrArray) images = NULL;
@@ -385,8 +405,6 @@ gs_appstream_refine_add_images (GsApp *app, AsScreenshot *ss, XbNode *screenshot
 	if (images == NULL) {
 		if (g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
 			return TRUE;
-		if (g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT))
-			return TRUE;
 		g_propagate_error (error, g_steal_pointer (&error_local));
 		return FALSE;
 	}
@@ -400,10 +418,45 @@ gs_appstream_refine_add_images (GsApp *app, AsScreenshot *ss, XbNode *screenshot
 		as_screenshot_add_image (ss, im);
 	}
 
+	*out_any_added = *out_any_added || images->len > 0;
+
 	/* success */
 	return TRUE;
 }
 
+static gboolean
+gs_appstream_refine_add_videos (GsApp *app,
+				AsScreenshot *ss,
+				XbNode *screenshot,
+				gboolean *out_any_added,
+				GError **error)
+{
+	g_autoptr(GError) error_local = NULL;
+	g_autoptr(GPtrArray) videos = NULL;
+
+	videos = xb_node_query (screenshot, "video", 0, &error_local);
+	if (videos == NULL) {
+		if (g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
+			return TRUE;
+		g_propagate_error (error, g_steal_pointer (&error_local));
+		return FALSE;
+	}
+	for (guint i = 0; i < videos->len; i++) {
+		XbNode *video = g_ptr_array_index (videos, i);
+		g_autoptr(AsVideo) vid = as_video_new ();
+		as_video_set_height (vid, xb_node_get_attr_as_uint (video, "height"));
+		as_video_set_width (vid, xb_node_get_attr_as_uint (video, "width"));
+		as_video_set_codec_kind (vid, as_video_codec_kind_from_string (xb_node_get_attr (video, "codec")));
+		as_video_set_container_kind (vid, as_video_container_kind_from_string (xb_node_get_attr (video, "container")));
+		as_video_set_url (vid, xb_node_get_text (video));
+		as_screenshot_add_video (ss, vid);
+	}
+
+	*out_any_added = *out_any_added || videos->len > 0;
+
+	return TRUE;
+}
+
 static gboolean
 gs_appstream_refine_add_screenshots (GsApp *app, XbNode *component, GError **error)
 {
@@ -415,17 +468,18 @@ gs_appstream_refine_add_screenshots (GsApp *app, XbNode *component, GError **err
 	if (screenshots == NULL) {
 		if (g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
 			return TRUE;
-		if (g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT))
-			return TRUE;
 		g_propagate_error (error, g_steal_pointer (&error_local));
 		return FALSE;
 	}
 	for (guint i = 0; i < screenshots->len; i++) {
 		XbNode *screenshot = g_ptr_array_index (screenshots, i);
 		g_autoptr(AsScreenshot) ss = as_screenshot_new ();
-		if (!gs_appstream_refine_add_images (app, ss, screenshot, error))
+		gboolean any_added = FALSE;
+		if (!gs_appstream_refine_add_images (app, ss, screenshot, &any_added, error) ||
+		    !gs_appstream_refine_add_videos (app, ss, screenshot, &any_added, error))
 			return FALSE;
-		gs_app_add_screenshot (app, ss);
+		if (any_added)
+			gs_app_add_screenshot (app, ss);
 	}
 
 	/* FIXME: move into no refine flags section? */
@@ -447,8 +501,6 @@ gs_appstream_refine_add_provides (GsApp *app, XbNode *component, GError **error)
 	if (provides == NULL) {
 		if (g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
 			return TRUE;
-		if (g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT))
-			return TRUE;
 		g_propagate_error (error, g_steal_pointer (&error_local));
 		return FALSE;
 	}
@@ -552,8 +604,6 @@ gs_appstream_copy_metadata (GsApp *app, XbNode *component, GError **error)
 	if (values == NULL) {
 		if (g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
 			return TRUE;
-		if (g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT))
-			return TRUE;
 		g_propagate_error (error, g_steal_pointer (&error_local));
 		return FALSE;
 	}
@@ -592,8 +642,7 @@ gs_appstream_refine_app_updates (GsApp *app,
 				 gs_app_get_id (app));
 	releases_inst = xb_silo_query (silo, xpath, 0, &error_local);
 	if (releases_inst == NULL) {
-		if (!g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_NOT_FOUND) &&
-		    !g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT)) {
+		if (!g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_NOT_FOUND)) {
 			g_propagate_error (error, g_steal_pointer (&error_local));
 			return FALSE;
 		}
@@ -612,8 +661,6 @@ gs_appstream_refine_app_updates (GsApp *app,
 	if (releases == NULL) {
 		if (g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
 			return TRUE;
-		if (g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT))
-			return TRUE;
 		g_propagate_error (error, g_steal_pointer (&error_local));
 		return FALSE;
 	}
@@ -713,8 +760,6 @@ gs_appstream_refine_add_version_history (GsApp *app, XbNode *component, GError *
 	if (releases == NULL) {
 		if (g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
 			return TRUE;
-		if (g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT))
-			return TRUE;
 		g_propagate_error (error, g_steal_pointer (&error_local));
 		return FALSE;
 	}
@@ -835,9 +880,6 @@ gs_appstream_refine_app_content_rating (GsApp *app,
 	if (content_attributes == NULL &&
 	    g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_NOT_FOUND)) {
 		g_clear_error (&error_local);
-	} else if (content_attributes == NULL &&
-		 g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT)) {
-		return TRUE;
 	} else if (content_attributes == NULL) {
 		g_propagate_error (error, g_steal_pointer (&error_local));
 		return FALSE;
@@ -867,8 +909,6 @@ gs_appstream_refine_app_content_ratings (GsApp *app,
 	if (content_ratings == NULL) {
 		if (g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
 			return TRUE;
-		if (g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT))
-			return TRUE;
 		g_propagate_error (error, g_steal_pointer (&error_local));
 		return FALSE;
 	}
@@ -897,14 +937,14 @@ gs_appstream_refine_app_relation (GsApp           *app,
 		as_relation_set_kind (relation, kind);
 
 		if (g_str_equal (item_kind, "control")) {
-			/* https://www.freedesktop.org/software/appstream/docs/chap-Metadata.html#tag-requires-recommends-control */
+			/* https://www.freedesktop.org/software/appstream/docs/chap-Metadata.html#tag-relations-control */
 			as_relation_set_item_kind (relation, AS_RELATION_ITEM_KIND_CONTROL);
 			as_relation_set_value_control_kind (relation, as_control_kind_from_string (xb_node_get_text (child)));
 		} else if (g_str_equal (item_kind, "display_length")) {
 			AsDisplayLengthKind display_length_kind;
 			const gchar *compare;
 
-			/* https://www.freedesktop.org/software/appstream/docs/chap-Metadata.html#tag-requires-recommends-display_length */
+			/* https://www.freedesktop.org/software/appstream/docs/chap-Metadata.html#tag-relations-display_length */
 			as_relation_set_item_kind (relation, AS_RELATION_ITEM_KIND_DISPLAY_LENGTH);
 
 			compare = xb_node_get_attr (child, "compare");
@@ -936,42 +976,34 @@ gs_appstream_refine_app_relations (GsApp     *app,
                                    XbNode    *component,
                                    GError   **error)
 {
-	g_autoptr(GPtrArray) recommends = NULL;
-	g_autoptr(GPtrArray) requires = NULL;
-	g_autoptr(GError) error_local = NULL;
+	const struct {
+		const gchar *element_name;
+		AsRelationKind relation_kind;
+	} relation_types[] = {
+#if AS_CHECK_VERSION(0, 15, 0)
+		{ "supports", AS_RELATION_KIND_SUPPORTS },
+#endif
+		{ "recommends", AS_RELATION_KIND_RECOMMENDS },
+		{ "requires", AS_RELATION_KIND_REQUIRES },
+	};
 
-	/* find any recommends */
-	recommends = xb_node_query (component, "recommends", 0, &error_local);
-	if (recommends == NULL) {
-		if (g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
-			return TRUE;
-		if (g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT))
-			return TRUE;
-		g_propagate_error (error, g_steal_pointer (&error_local));
-		return FALSE;
-	}
+	for (gsize i = 0; i < G_N_ELEMENTS (relation_types); i++) {
+		g_autoptr(GPtrArray) relations = NULL;
+		g_autoptr(GError) error_local = NULL;
 
-	for (guint i = 0; i < recommends->len; i++) {
-		XbNode *recommend = g_ptr_array_index (recommends, i);
-		if (!gs_appstream_refine_app_relation (app, recommend, AS_RELATION_KIND_RECOMMENDS, error))
+		/* find any instances of this @element_name */
+		relations = xb_node_query (component, relation_types[i].element_name, 0, &error_local);
+		if (relations == NULL &&
+		    !g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_NOT_FOUND)) {
+			g_propagate_error (error, g_steal_pointer (&error_local));
 			return FALSE;
-	}
+		}
 
-	/* find any requires */
-	requires = xb_node_query (component, "requires", 0, &error_local);
-	if (requires == NULL) {
-		if (g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
-			return TRUE;
-		if (g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT))
-			return TRUE;
-		g_propagate_error (error, g_steal_pointer (&error_local));
-		return FALSE;
-	}
-
-	for (guint i = 0; i < requires->len; i++) {
-		XbNode *require = g_ptr_array_index (requires, i);
-		if (!gs_appstream_refine_app_relation (app, require, AS_RELATION_KIND_REQUIRES, error))
-			return FALSE;
+		for (guint j = 0; relations != NULL && j < relations->len; j++) {
+			XbNode *relation = g_ptr_array_index (relations, j);
+			if (!gs_appstream_refine_app_relation (app, relation, relation_types[i].relation_kind, error))
+				return FALSE;
+		}
 	}
 
 	return TRUE;
@@ -991,6 +1023,11 @@ gs_appstream_refine_app (GsPlugin *plugin,
 	g_autoptr(GPtrArray) launchables = NULL;
 	g_autoptr(XbNode) req = NULL;
 
+	/* The 'plugin' can be NULL, when creating app for --show-metainfo */
+	g_return_val_if_fail (GS_IS_APP (app), FALSE);
+	g_return_val_if_fail (XB_IS_SILO (silo), FALSE);
+	g_return_val_if_fail (XB_IS_NODE (component), FALSE);
+
 	/* is compatible */
 	req = xb_node_query_first (component,
 				   "requires/id[@type='id']"
@@ -1408,32 +1445,29 @@ gs_appstream_silo_search_component (GPtrArray *array, XbNode *component, const g
 	return matches_sum;
 }
 
-gboolean
-gs_appstream_search (GsPlugin *plugin,
-		     XbSilo *silo,
-		     const gchar * const *values,
-		     GsAppList *list,
-		     GCancellable *cancellable,
-		     GError **error)
+typedef struct {
+	AsSearchTokenMatch	match_value;
+	const gchar		*xpath;
+} Query;
+
+static gboolean
+gs_appstream_do_search (GsPlugin *plugin,
+			XbSilo *silo,
+			const gchar * const *values,
+			const Query queries[],
+			GsAppList *list,
+			GCancellable *cancellable,
+			GError **error)
 {
 	g_autoptr(GError) error_local = NULL;
 	g_autoptr(GPtrArray) array = g_ptr_array_new_with_free_func ((GDestroyNotify) gs_appstream_search_helper_free);
 	g_autoptr(GPtrArray) components = NULL;
 	g_autoptr(GTimer) timer = g_timer_new ();
-	const struct {
-		AsSearchTokenMatch	match_value;
-		const gchar		*xpath;
-	} queries[] = {
-		{ AS_SEARCH_TOKEN_MATCH_MIMETYPE,	"mimetypes/mimetype[text()~=stem(?)]" },
-		{ AS_SEARCH_TOKEN_MATCH_PKGNAME,	"pkgname[text()~=stem(?)]" },
-		{ AS_SEARCH_TOKEN_MATCH_SUMMARY,	"summary[text()~=stem(?)]" },
-		{ AS_SEARCH_TOKEN_MATCH_NAME,	"name[text()~=stem(?)]" },
-		{ AS_SEARCH_TOKEN_MATCH_KEYWORD,	"keywords/keyword[text()~=stem(?)]" },
-		{ AS_SEARCH_TOKEN_MATCH_ID,	"id[text()~=stem(?)]" },
-		{ AS_SEARCH_TOKEN_MATCH_ID,	"launchable[text()~=stem(?)]" },
-		{ AS_SEARCH_TOKEN_MATCH_ORIGIN,	"../components[@origin~=stem(?)]" },
-		{ AS_SEARCH_TOKEN_MATCH_NONE,	NULL }
-	};
+
+	g_return_val_if_fail (GS_IS_PLUGIN (plugin), FALSE);
+	g_return_val_if_fail (XB_IS_SILO (silo), FALSE);
+	g_return_val_if_fail (values != NULL, FALSE);
+	g_return_val_if_fail (GS_IS_APP_LIST (list), FALSE);
 
 	/* add some weighted queries */
 	for (guint i = 0; queries[i].xpath != NULL; i++) {
@@ -1454,8 +1488,6 @@ gs_appstream_search (GsPlugin *plugin,
 	if (components == NULL) {
 		if (g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
 			return TRUE;
-		if (g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT))
-			return TRUE;
 		g_propagate_error (error, g_steal_pointer (&error_local));
 		return FALSE;
 	}
@@ -1503,6 +1535,48 @@ gs_appstream_search (GsPlugin *plugin,
 	return TRUE;
 }
 
+/* This tokenises and stems @values internally for comparison against the
+ * already-stemmed tokens in the libxmlb silo */
+gboolean
+gs_appstream_search (GsPlugin *plugin,
+		     XbSilo *silo,
+		     const gchar * const *values,
+		     GsAppList *list,
+		     GCancellable *cancellable,
+		     GError **error)
+{
+	const Query queries[] = {
+		{ AS_SEARCH_TOKEN_MATCH_MIMETYPE,	"mimetypes/mimetype[text()~=stem(?)]" },
+		{ AS_SEARCH_TOKEN_MATCH_PKGNAME,	"pkgname[text()~=stem(?)]" },
+		{ AS_SEARCH_TOKEN_MATCH_SUMMARY,	"summary[text()~=stem(?)]" },
+		{ AS_SEARCH_TOKEN_MATCH_NAME,	"name[text()~=stem(?)]" },
+		{ AS_SEARCH_TOKEN_MATCH_KEYWORD,	"keywords/keyword[text()~=stem(?)]" },
+		{ AS_SEARCH_TOKEN_MATCH_ID,	"id[text()~=stem(?)]" },
+		{ AS_SEARCH_TOKEN_MATCH_ID,	"launchable[text()~=stem(?)]" },
+		{ AS_SEARCH_TOKEN_MATCH_ORIGIN,	"../components[@origin~=stem(?)]" },
+		{ AS_SEARCH_TOKEN_MATCH_NONE,	NULL }
+	};
+
+	return gs_appstream_do_search (plugin, silo, values, queries, list, cancellable, error);
+}
+
+gboolean
+gs_appstream_search_developer_apps (GsPlugin *plugin,
+				    XbSilo *silo,
+				    const gchar * const *values,
+				    GsAppList *list,
+				    GCancellable *cancellable,
+				    GError **error)
+{
+	const Query queries[] = {
+		{ AS_SEARCH_TOKEN_MATCH_PKGNAME,	"developer_name[text()~=stem(?)]" },
+		{ AS_SEARCH_TOKEN_MATCH_SUMMARY,	"project_group[text()~=stem(?)]" },
+		{ AS_SEARCH_TOKEN_MATCH_NONE,		NULL }
+	};
+
+	return gs_appstream_do_search (plugin, silo, values, queries, list, cancellable, error);
+}
+
 gboolean
 gs_appstream_add_category_apps (GsPlugin *plugin,
 				XbSilo *silo,
@@ -1512,7 +1586,11 @@ gs_appstream_add_category_apps (GsPlugin *plugin,
 				GError **error)
 {
 	GPtrArray *desktop_groups;
-	g_autoptr(GError) error_local = NULL;
+
+	g_return_val_if_fail (GS_IS_PLUGIN (plugin), FALSE);
+	g_return_val_if_fail (XB_IS_SILO (silo), FALSE);
+	g_return_val_if_fail (GS_IS_CATEGORY (category), FALSE);
+	g_return_val_if_fail (GS_IS_APP_LIST (list), FALSE);
 
 	desktop_groups = gs_category_get_desktop_groups (category);
 	if (desktop_groups->len == 0) {
@@ -1524,6 +1602,7 @@ gs_appstream_add_category_apps (GsPlugin *plugin,
 		g_autofree gchar *xpath = NULL;
 		g_auto(GStrv) split = g_strsplit (desktop_group, "::", -1);
 		g_autoptr(GPtrArray) components = NULL;
+		g_autoptr(GError) error_local = NULL;
 
 		/* generate query */
 		if (g_strv_length (split) == 1) {
@@ -1539,9 +1618,7 @@ gs_appstream_add_category_apps (GsPlugin *plugin,
 		components = xb_silo_query (silo, xpath, 0, &error_local);
 		if (components == NULL) {
 			if (g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
-				return TRUE;
-			if (g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT))
-				return TRUE;
+				continue;
 			g_propagate_error (error, g_steal_pointer (&error_local));
 			return FALSE;
 		}
@@ -1591,8 +1668,6 @@ gs_appstream_count_component_for_groups (XbSilo      *silo,
 	if (array == NULL) {
 		if (g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
 			return 0;
-		if (g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT))
-			return 0;
 		g_warning ("%s", error_local->message);
 		return 0;
 	}
@@ -1602,11 +1677,14 @@ gs_appstream_count_component_for_groups (XbSilo      *silo,
 /* we're not actually adding categories here, we're just setting the number of
  * applications available in each category */
 gboolean
-gs_appstream_add_categories (XbSilo *silo,
-			     GPtrArray *list,
-			     GCancellable *cancellable,
-			     GError **error)
+gs_appstream_refine_category_sizes (XbSilo        *silo,
+                                    GPtrArray     *list,
+                                    GCancellable  *cancellable,
+                                    GError       **error)
 {
+	g_return_val_if_fail (XB_IS_SILO (silo), FALSE);
+	g_return_val_if_fail (list != NULL, FALSE);
+
 	for (guint j = 0; j < list->len; j++) {
 		GsCategory *parent = GS_CATEGORY (g_ptr_array_index (list, j));
 		GPtrArray *children = gs_category_get_children (parent);
@@ -1617,12 +1695,12 @@ gs_appstream_add_categories (XbSilo *silo,
 			for (guint k = 0; k < groups->len; k++) {
 				const gchar *group = g_ptr_array_index (groups, k);
 				guint cnt = gs_appstream_count_component_for_groups (silo, group);
-				for (guint l = 0; l < cnt; l++) {
-					gs_category_increment_size (parent);
+				if (cnt > 0) {
+					gs_category_increment_size (parent, cnt);
 					if (children->len > 1) {
 						/* Parent category has multiple groups, so increment
 						 * each group's size too */
-						gs_category_increment_size (cat);
+						gs_category_increment_size (cat, cnt);
 					}
 				}
 			}
@@ -1632,6 +1710,42 @@ gs_appstream_add_categories (XbSilo *silo,
 	return TRUE;
 }
 
+gboolean
+gs_appstream_add_installed (GsPlugin      *plugin,
+                            XbSilo        *silo,
+                            GsAppList     *list,
+                            GCancellable  *cancellable,
+                            GError       **error)
+{
+	g_autoptr(GPtrArray) components = NULL;
+	g_autoptr(GError) local_error = NULL;
+
+	g_return_val_if_fail (GS_IS_PLUGIN (plugin), FALSE);
+	g_return_val_if_fail (XB_IS_SILO (silo), FALSE);
+	g_return_val_if_fail (GS_IS_APP_LIST (list), FALSE);
+
+	/* get all installed appdata files (notice no 'components/' prefix...) */
+	components = xb_silo_query (silo, "component/description/..", 0, NULL);
+	if (components == NULL)
+		return TRUE;
+
+	for (guint i = 0; i < components->len; i++) {
+		XbNode *component = g_ptr_array_index (components, i);
+		g_autoptr(GsApp) app = gs_appstream_create_app (plugin, silo, component, error);
+		if (app == NULL)
+			return FALSE;
+
+		/* Can get cached GsApp, which has the state already updated */
+		if (gs_app_get_state (app) != GS_APP_STATE_UPDATABLE &&
+		    gs_app_get_state (app) != GS_APP_STATE_UPDATABLE_LIVE)
+			gs_app_set_state (app, GS_APP_STATE_INSTALLED);
+		gs_app_set_scope (app, AS_COMPONENT_SCOPE_SYSTEM);
+		gs_app_list_add (list, app);
+	}
+
+	return TRUE;
+}
+
 gboolean
 gs_appstream_add_popular (XbSilo *silo,
 			  GsAppList *list,
@@ -1641,6 +1755,9 @@ gs_appstream_add_popular (XbSilo *silo,
 	g_autoptr(GError) error_local = NULL;
 	g_autoptr(GPtrArray) array = NULL;
 
+	g_return_val_if_fail (XB_IS_SILO (silo), FALSE);
+	g_return_val_if_fail (GS_IS_APP_LIST (list), FALSE);
+
 	/* find out how many packages are in each category */
 	array = xb_silo_query (silo,
 			       "components/component/kudos/"
@@ -1649,8 +1766,6 @@ gs_appstream_add_popular (XbSilo *silo,
 	if (array == NULL) {
 		if (g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
 			return TRUE;
-		if (g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT))
-			return TRUE;
 		g_propagate_error (error, g_steal_pointer (&error_local));
 		return FALSE;
 	}
@@ -1680,6 +1795,10 @@ gs_appstream_add_recent (GsPlugin *plugin,
 	g_autoptr(GError) error_local = NULL;
 	g_autoptr(GPtrArray) array = NULL;
 
+	g_return_val_if_fail (GS_IS_PLUGIN (plugin), FALSE);
+	g_return_val_if_fail (XB_IS_SILO (silo), FALSE);
+	g_return_val_if_fail (GS_IS_APP_LIST (list), FALSE);
+
 	/* use predicate conditions to the max */
 	xpath = g_strdup_printf ("components/component/releases/"
 				 "release[@timestamp>%" G_GUINT64_FORMAT "]/../..",
@@ -1688,8 +1807,6 @@ gs_appstream_add_recent (GsPlugin *plugin,
 	if (array == NULL) {
 		if (g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
 			return TRUE;
-		if (g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT))
-			return TRUE;
 		g_propagate_error (error, g_steal_pointer (&error_local));
 		return FALSE;
 	}
@@ -1720,6 +1837,10 @@ gs_appstream_add_alternates (XbSilo *silo,
 	g_autoptr(GPtrArray) ids = NULL;
 	g_autoptr(GString) xpath = g_string_new (NULL);
 
+	g_return_val_if_fail (XB_IS_SILO (silo), FALSE);
+	g_return_val_if_fail (GS_IS_APP (app), FALSE);
+	g_return_val_if_fail (GS_IS_APP_LIST (list), FALSE);
+
 	/* probably a package we know nothing about */
 	if (gs_app_get_id (app) == NULL)
 		return TRUE;
@@ -1750,8 +1871,6 @@ gs_appstream_add_alternates (XbSilo *silo,
 	if (ids == NULL) {
 		if (g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
 			return TRUE;
-		if (g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT))
-			return TRUE;
 		g_propagate_error (error, g_steal_pointer (&error_local));
 		return FALSE;
 	}
@@ -1770,25 +1889,24 @@ gs_appstream_add_alternates (XbSilo *silo,
 	return TRUE;
 }
 
-gboolean
-gs_appstream_add_featured (XbSilo *silo,
-			   GsAppList *list,
-			   GCancellable *cancellable,
-			   GError **error)
+static gboolean
+gs_appstream_add_featured_with_query (XbSilo *silo,
+				      const gchar *query,
+				      GsAppList *list,
+				      GCancellable *cancellable,
+				      GError **error)
 {
 	g_autoptr(GError) error_local = NULL;
 	g_autoptr(GPtrArray) array = NULL;
 
+	g_return_val_if_fail (XB_IS_SILO (silo), FALSE);
+	g_return_val_if_fail (GS_IS_APP_LIST (list), FALSE);
+
 	/* find out how many packages are in each category */
-	array = xb_silo_query (silo,
-			       "components/component/custom/value[@key='GnomeSoftware::FeatureTile']/../..|"
-			       "components/component/custom/value[@key='GnomeSoftware::FeatureTile-css']/../..",
-			       0, &error_local);
+	array = xb_silo_query (silo, query, 0, &error_local);
 	if (array == NULL) {
 		if (g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
 			return TRUE;
-		if (g_error_matches (error_local, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT))
-			return TRUE;
 		g_propagate_error (error, g_steal_pointer (&error_local));
 		return FALSE;
 	}
@@ -1807,6 +1925,41 @@ gs_appstream_add_featured (XbSilo *silo,
 	return TRUE;
 }
 
+gboolean
+gs_appstream_add_featured (XbSilo *silo,
+			   GsAppList *list,
+			   GCancellable *cancellable,
+			   GError **error)
+{
+	const gchar *query = "components/component/custom/value[@key='GnomeSoftware::FeatureTile']/../..|"
+			     "components/component/custom/value[@key='GnomeSoftware::FeatureTile-css']/../..";
+	return gs_appstream_add_featured_with_query (silo, query, list, cancellable, error);
+}
+
+gboolean
+gs_appstream_add_deployment_featured (XbSilo *silo,
+				      const gchar * const *deployments,
+				      GsAppList *list,
+				      GCancellable *cancellable,
+				      GError **error)
+{
+	g_autoptr(GString) query = g_string_new (NULL);
+	g_return_val_if_fail (XB_IS_SILO (silo), FALSE);
+	g_return_val_if_fail (deployments != NULL, FALSE);
+	g_return_val_if_fail (GS_IS_APP_LIST (list), FALSE);
+	for (guint ii = 0; deployments[ii] != NULL; ii++) {
+		g_autofree gchar *escaped = xb_string_escape (deployments[ii]);
+		if (escaped != NULL && *escaped != '\0') {
+			xb_string_append_union (query,
+				"components/component/custom/value[@key='GnomeSoftware::DeploymentFeatured'][text()='%s']/../..",
+				escaped);
+		}
+	}
+	if (!query->len)
+		return TRUE;
+	return gs_appstream_add_featured_with_query (silo, query->str, list, cancellable, error);
+}
+
 gboolean
 gs_appstream_url_to_app (GsPlugin *plugin,
 			 XbSilo *silo,
@@ -1820,6 +1973,11 @@ gs_appstream_url_to_app (GsPlugin *plugin,
 	g_autofree gchar *xpath = NULL;
 	g_autoptr(GPtrArray) components = NULL;
 
+	g_return_val_if_fail (GS_IS_PLUGIN (plugin), FALSE);
+	g_return_val_if_fail (XB_IS_SILO (silo), FALSE);
+	g_return_val_if_fail (GS_IS_APP_LIST (list), FALSE);
+	g_return_val_if_fail (url != NULL, FALSE);
+
 	/* not us */
 	scheme = gs_utils_get_url_scheme (url);
 	if (g_strcmp0 (scheme, "appstream") != 0)
@@ -1850,6 +2008,9 @@ gs_appstream_component_add_keyword (XbBuilderNode *component, const gchar *str)
 	g_autoptr(XbBuilderNode) keyword = NULL;
 	g_autoptr(XbBuilderNode) keywords = NULL;
 
+	g_return_if_fail (XB_IS_BUILDER_NODE (component));
+	g_return_if_fail (str != NULL);
+
 	/* create <keywords> if it does not already exist */
 	keywords = xb_builder_node_get_child (component, "keywords", NULL);
 	if (keywords == NULL)
@@ -1869,6 +2030,9 @@ gs_appstream_component_add_provide (XbBuilderNode *component, const gchar *str)
 	g_autoptr(XbBuilderNode) provide = NULL;
 	g_autoptr(XbBuilderNode) provides = NULL;
 
+	g_return_if_fail (XB_IS_BUILDER_NODE (component));
+	g_return_if_fail (str != NULL);
+
 	/* create <provides> if it does not already exist */
 	provides = xb_builder_node_get_child (component, "provides", NULL);
 	if (provides == NULL)
@@ -1888,6 +2052,9 @@ gs_appstream_component_add_category (XbBuilderNode *component, const gchar *str)
 	g_autoptr(XbBuilderNode) category = NULL;
 	g_autoptr(XbBuilderNode) categories = NULL;
 
+	g_return_if_fail (XB_IS_BUILDER_NODE (component));
+	g_return_if_fail (str != NULL);
+
 	/* create <categories> if it does not already exist */
 	categories = xb_builder_node_get_child (component, "categories", NULL);
 	if (categories == NULL)
@@ -1906,6 +2073,9 @@ gs_appstream_component_add_icon (XbBuilderNode *component, const gchar *str)
 {
 	g_autoptr(XbBuilderNode) icon = NULL;
 
+	g_return_if_fail (XB_IS_BUILDER_NODE (component));
+	g_return_if_fail (str != NULL);
+
 	/* create <icon>str</icon> if it does not already exist */
 	icon = xb_builder_node_get_child (component, "icon", NULL);
 	if (icon == NULL) {
@@ -1919,7 +2089,11 @@ gs_appstream_component_add_icon (XbBuilderNode *component, const gchar *str)
 void
 gs_appstream_component_add_extra_info (XbBuilderNode *component)
 {
-	const gchar *kind = xb_builder_node_get_attr (component, "type");
+	const gchar *kind;
+
+	g_return_if_fail (XB_IS_BUILDER_NODE (component));
+
+	kind = xb_builder_node_get_attr (component, "type");
 
 	/* add the gnome-software-specific 'Addon' group and ensure they
 	 * all have an icon set */
@@ -1965,9 +2139,14 @@ gs_appstream_component_add_extra_info (XbBuilderNode *component)
 void
 gs_appstream_component_fix_url (XbBuilderNode *component, const gchar *baseurl)
 {
-	const gchar *text = xb_builder_node_get_text (component);
+	const gchar *text;
 	g_autofree gchar *url = NULL;
 
+	g_return_if_fail (XB_IS_BUILDER_NODE (component));
+	g_return_if_fail (baseurl != NULL);
+
+	text = xb_builder_node_get_text (component);
+
 	if (text == NULL)
 		return;
 
